from sawa.process import Process
from sawa.enum.file_error import FileError
import os
import sys
script_dir = os.path.dirname(__file__)

num_dict = {
    "꧐": "0",
    "꧑": "1",
    "꧒": "2",
    "꧓": "3",
    "꧔": "4",
    "꧕": "5",
    "꧖": "6",
    "꧗": "7",
    "꧘": "8",
    "꧙": "9",
}

keyword_dict = {
    "ꦧꦼꦤꦼꦂ": "True", # bener
    "ꦱꦭꦃ": "False", # salah
    "ꦱꦸꦮꦸꦁ": "None", # suwung
    "ꦢꦢꦶ": "as", # dadi
    "ꦱꦶꦢ": "assert", # sida
    "ꦲꦺꦴꦫꦩꦡꦸꦏ꧀": "async", # oramathuk
    "ꦭꦤ꧀": "and", #lan
    "ꦤꦸꦁꦒꦸ": "await", #nunggu
    "ꦭꦺꦫꦺꦤ꧀": "break", #leren
    "ꦏꦼꦭꦱ꧀": "class", #kelas
    "ꦠꦼꦫꦸꦱ꧀ꦱꦏꦺ": "continue", #terusake
    "ꦥ꦳ꦸꦁꦱꦶ": "def", #fungsi
    "ꦧꦸꦱꦏ꧀": "del", #busak
    "ꦲꦸꦠꦮꦭꦶꦪꦤꦺ": "elif", #utawaliyane
    "ꦭꦶꦪꦤꦺ": "else", #liyane
    "ꦏꦼꦗꦧ": "except", #kejaba
    "ꦫꦩ꧀ꦥꦸꦁꦔꦤ꧀": "finally", #rampungane
    "ꦏꦁꦒꦺꦴ": "for", #kanggo
    "ꦱꦏ": "from", #saka
    "ꦗꦒꦢ꧀": "global", #jagad
    "ꦪꦺꦤ꧀": "if", #yen
    "ꦗꦸꦥꦸꦏ꧀": "import", #jupuk
    "ꦲꦶꦁ": "in", #ing
    "ꦪꦲꦶꦏꦸ": "is", #yaiku
    "ꦧꦶꦪꦤ꧀ꦠꦸ": "lambda", #biyantu
    "ꦢꦸꦢꦸ": "not", #dudu
    "ꦲꦺꦴꦫꦭꦺꦴꦏꦭ꧀": "nonlocal", #oralokal
    "ꦲꦸꦠꦮ": "or", #utawa
    "ꦭꦶꦮꦠ꧀ꦠꦶ": "pass", #liwati
    "ꦲꦁꦏꦠ꧀": "raise", #angkat
    "ꦧꦭꦶꦏ꧀": "return", #balik
    "ꦕꦺꦴꦧ": "try", #coba
    "ꦱꦮꦼꦠꦮꦶꦱ꧀": "while", #sawetawis
    "ꦏꦫꦺꦴ": "with", #karo
    "ꦏꦱꦶꦭ꧀": "yield", #kasil
    "ꦭꦺꦴꦒꦶꦱ꧀": "bool", #logis
    "ꦲꦁꦏ": "int", #angka
    "ꦣꦺꦱꦶꦩꦭ꧀": "float", #dhesimal
    "ꦲꦸꦏꦫ": "string", #ukara
    "__ꦲꦺꦴꦧ꧀ꦗꦺꦏ꧀__": "__dict__", # __objèk__
}

function_dict = {
    "ꦩꦸꦠ꧀ꦭꦏ꧀": "abs", #mutlak
    "ꦲꦥꦸꦱ꧀ꦲꦠꦿꦶꦧꦸꦠ꧀": "delattr", #apusatribut
    "ꦕꦩ꧀ꦥꦸꦂ": "hash", #campur
    "ꦱꦏ꧀ꦭꦺꦧꦠ꧀": "memoryview", #saklebat
    "ꦲꦺꦴꦩ꧀ꦧꦾꦺꦴꦏ꧀": "set", #ombrok
    "ꦏꦧꦺꦃ": "all", #kabeh
    "ꦧꦲꦸꦱꦱ꧀ꦠꦿ": "dict", #bausastra
    "ꦥꦶꦠꦸꦭꦸꦁ": "help", #pitulung
    "ꦩꦶꦤꦶꦩꦭ꧀": "min", # minimal
    "ꦒꦺꦠꦂ": "getattr", # gètar
    "ꦱꦺꦠꦂ": "setattr", # sètar
    "ꦏꦁ": "any", # kang
    "ꦣꦶꦂ": "dir", # dhir
    "ꦲꦺꦏ꦳꧀": "hex", #hèks
    "ꦧꦕꦸꦠ꧀": "next", #bacut
    "ꦲꦶꦫꦶꦱ꧀ꦱꦤ꧀": "slice", # irisan
    "ꦲꦱ꧀ꦏꦶ": "ascii", # aski
    "ꦣꦶꦥ꦳꧀ꦩꦺꦴꦢ꧀": "divmod", # dhifmod
    "ꦲꦶꦣꦶ": "id", # idhi
    "ꦲꦺꦴꦧ꧀ꦗꦺꦏ꧀": "object", # objèk
    "ꦲꦸꦫꦸꦠ꧀": "sorted", # urut
    "ꦧꦶꦤ꧀": "bin", # bin
    "ꦲꦺꦤꦸꦩꦼꦂꦫꦺꦠ꧀": "enumerate", # enumeret
    "ꦠꦸꦭꦶꦱ꧀": "input", #tulis
    "ꦲꦺꦴꦏ꧀ꦠ": "oct", # okta
    "ꦱꦶꦒꦼꦒ꧀": "staticmethod", #sigeg
    "ꦭꦺꦴꦒꦶꦱ꧀": "bool", # logis
    "ꦲꦺꦥ꦳ꦭ꧀": "eval", # efal
    "ꦲꦁꦏ": "int", # angka
    "ꦧꦸꦏꦏ꧀": "open", # bukak
    "ꦱꦼꦫꦠ꧀": "str", # serat
    "ꦊꦏꦱ꧀ꦩꦤꦺꦃ": "breakpoint", #lekasmanèh
    "ꦲꦗꦂ": "exec", # ajar
    "ꦧꦭꦤꦺ": "isinstance", #balané
    "ꦲꦺꦴꦫꦼꦢ꧀": "ord", #ored
    "ꦗꦸꦩ꧀ꦭꦃ": "sum", #jumlah
    "ꦧꦶꦠꦼꦫꦺ": "bytearray", #bitere
    "ꦥ꦳ꦶꦭ꧀ꦠꦼꦂ": "filter", #filter
    "ꦲꦤ꧀ꦣꦃꦲꦤ꧀": "issubclass", #andhahan
    "ꦏ꧀ꦮꦱ": "pow", #kwasa
    "ꦲꦸꦠꦩ": "super", #utama
    "ꦧꦪ꧀ꦠ": "bytes", #bayta
    "ꦣꦺꦱꦶꦩꦭ꧀": "float", #dhèsimal
    "ꦥꦫ": "iter", # para
    "ꦠꦸꦥꦼꦭ꧀": "tuple", # tupel
    "ꦔꦸꦚ꧀ꦢꦁ": "callable", # ngundang
    "ꦮꦸꦗꦸꦢ꧀": "format", # wujud
    "ꦢꦮ": "len", # dawa
    "ꦏꦒꦸꦁꦔꦤꦺ": "property", # kagungané
    "ꦩꦺꦴꦣꦺꦭ꧀": "type", # modhèl
    "ꦕꦂ": "chr", #car
    "ꦧꦼꦏꦸꦮꦤ꧀": "frozenset", #bekuwan
    "ꦥꦿꦠꦺꦭꦤ꧀": "list", #pratélan
    "ꦲꦤ꧀ꦠꦫ": "range", # antara
    "ꦥ꦳ꦉꦱ꧀": "vars", # fares
    "ꦩꦺꦠꦺꦴꦢ꧀ꦏꦼꦭꦱ꧀": "classmethod", #metodkelas
    "ꦗꦸꦥꦸꦏ꧀ꦲꦠꦼꦂ": "getattr", #jupuk ater
    "ꦥꦿꦶꦧꦸꦩꦶ": "locals", #pribumi
    "ꦫꦺꦥꦼꦂ": "repr", #reper
    "ꦗꦶꦥ꧀": "zip", #jip
    "ꦏꦺꦴꦩ꧀ꦥꦺꦭ꧀": "compile", #kompel
    "ꦱꦗꦒꦢ꧀": "globals", #sajagad
    "ꦥꦼꦠ": "map", #peta
    "ꦮꦭꦶꦏ꧀ꦲꦤꦺ": "reversed", #walikane
    "__ꦗꦸꦥꦸꦏ꧀__": "__import__", # __jupuk__
    "__ꦊꦏꦱ꧀__": "__init__", # __lekas__
    "__ꦲꦺꦴꦧ꧀ꦗꦺꦏ꧀__": "__dict__", # __objèk__
    "ꦩꦤꦺꦏ": "complex", #maneka
    "ꦒꦣꦃ": "hasattr", #gadhah
    "ꦥꦺꦴꦭ꧀": "max", #pol
    "ꦮꦸꦠꦸꦃ": "round", #wutuh
    "ꦣꦺꦮꦺ": "self", #dhewe
}

def compile(file_name):
    py_content = ''
    # Appending print function code to the file
    with open(os.path.join(script_dir, 'include.py'), 'r') as file:
        py_content += file.read()

    # Reading file to process the text
    with open(file_name, 'r') as file:
        # Process all content of the file
        py_content += Process(file, num_dict, keyword_dict, function_dict).process()

    # Splitting file name to remove existing extension in order to add python extension
    file_split = file_name.split(".")
    # Name of the python file to be created
    py_file = file_split[0] + ".py"
    # Creating the python file
    f = open(py_file, "w")
    # And then writing content to the python file
    f.write(py_content)

def main(file_name):
    if os.path.isdir(file_name):
        # If the file name is directory then it will search all the file with *.ꦱꦮ and process all file
        for flnm in os.listdir(file_name):
            if flnm.endswith('.ꦱꦮ'):
                compile(flnm)
                print(' \r\n ')
    elif os.path.isfile(file_name):
        compile(file_name)
    else:
        sys.exit("'" + file_name + "'" + " " + FileError.INVALID_FILE.value)




def replace_en_num(number: any) -> str:
    """
    Replace en number with jw number
    :param number: any
    :return: text: str
    """
    text = str(number)
    num_dict = {"0": "꧐", "1": "꧑", "2": "꧒", "3": "꧓", "4": "꧔", "5": "꧕", "6": "꧖", "7": "꧗", "8": "꧘", "9": "꧙"}
    for num in num_dict.keys():
        if num in text:
            text = text.replace(num, num_dict[num])
    return text



#!/usr/bin/env python3
'''
ꦱꦮ is an open source programming language, an interpreter to be precise, where you can write Python code using Javanese character.
https://github.com/lantip/sawa/docs/
Licensed under ABRMS License
Copyright (c) 2021 ꦱꦮ  
'''
import sys
import os
import argparse
import sawa.main as mn
from subprocess import call

def error(_error, message):
    """ Print errors to stdout
    """
    print("[-] {}: {}".format(_error, message))
    sys.exit(0)

def main():
    parser = argparse.ArgumentParser(description='ꦒꦩ꧀ꦧꦂꦥ꦳ꦶꦧꦺꦴꦤꦕꦶ')
    parser.add_argument('name', type=str, help='filename with extension .ꦱꦮ')
    args = parser.parse_args()

    if not 'ꦱꦮ' in args.name:
        error("Error",
                  "Please use ꦱꦮ as extension.")
    file_name = args.name.split('.')
    if os.path.isfile(file_name[0]+'.py'):
        os.remove(file_name[0]+'.py')
    mn.main(args.name)
    call('python3 %s.py' % (file_name[0]), shell=True)
    os.remove(file_name[0]+'.py')


def run_as_command():
    version = ".".join(str(v) for v in sys.version_info[:2])
    if float(version) < 3.6:
        print("[-] ꦱꦮ mbutuhaké Python vèrsi 3.6 munggah.")
        sys.exit(0)

    main()


if __name__ == '__main__':
    main()



import sys
from sawa.enum.effected import Effected
from sawa.enum.token_error import TokenError
import sawa.util as util


class Process:

    def __init__(self, file, num_dict: dict, keyword_dict: dict, function_dict: dict):
        self.file = file
        self.num_dict = num_dict
        self.keyword_dict = keyword_dict
        self.function_dict = function_dict
        self.line = ''
        self.buffer = ''
        self.token = ''
        self.las_symbol = ''
        self.next_symbol = ''
        self.current_symbol = ''
        self.effected = Effected.AS_NONE
        self.line_no = 0
        self.char_no = 0

        self.operators = '=!<>+-%/*^'
        self.symbols = '#\"\':;.,@'
        self.brackets = '()[]{}'
        self.all_symbols = self.operators + self.symbols + self.brackets

    def report_error(self, error: TokenError) -> None:
        error_str = error.value + " " + " ꦤꦁꦧꦫꦶꦱ꧀ꦤꦺꦴꦩꦼꦂ " \
                    + util.replace_en_num(self.line_no) + ":" + util.replace_en_num(self.char_no)
        error_str = self.line + '\n' + error_str
        sys.exit(error_str)

    def change_number(self) -> None:
        if self.token is None or self.token == '':
            return
        new_token = ''
        for c in self.token:
            if c in self.num_dict.keys():
                new_token += self.num_dict[c]
            else:
                self.report_error(TokenError.INVALID_TOKEN)
        self.token = new_token

    def change_keyword(self) -> bool:
        if self.token is None or self.token == '':
            return False

        # If the token is a javanese number it will replace english number
        if self.token[0] in self.num_dict.keys():
            self.change_number()
            return True
            # return ''.join([num_dict[c] for c in token]), True

        # If the token matches any python keywords it replaces that keywords
        if self.token in self.keyword_dict.keys():
            self.token = self.keyword_dict[self.token]
            if self.token == 'from' or self.token == 'import':
                pass
            elif self.token == 'if' or self.token == 'elif' or self.token == 'else' or self.token == 'while' or self.token == 'for':
                pass
            elif self.token == 'def' or self.token == 'lamda':
                pass
            elif self.token == 'class':
                pass
            return True

        # If nothing matches it returns token as it
        return False

    def change_function(self) -> bool:
        if self.token is None or self.token == '':
            return False

        # If the token matches any python in built it replaces that function name
        if self.token in self.function_dict.keys():
            self.token = self.function_dict[self.token]
            return True

        # If nothing matches it returns false
        return False

    def get_next_symbol(self) -> bool:
        self.next_symbol = ''
        for character in self.line[self.char_no:]:
            if character == " " or character == "\n" or character in self.all_symbols:
                if character != " " and character != "\n":
                    self.next_symbol = character
                    return True
            else:
                return False

        return False

    def process_line(self):

        py_line = ''  # new line to store all characters and processed buffer or buffer

        self.buffer = ''  # Store buffer
        self.char_no = 0  # store the characters position
        # self.effected = Effected.AS_NONE

        for character in self.line:
            self.char_no += 1
            if character == " " or character == "\n" or character in self.all_symbols:
                special_character = character  # Since it is special char

                if character != " " and character != "\n":
                    self.current_symbol = special_character

                if special_character == '\'' and self.effected is Effected.AS_NONE:
                    self.effected = Effected.AS_SINGLE_QUOTE_STRING
                elif special_character == '\'' and self.effected is Effected.AS_SINGLE_QUOTE_STRING:
                    self.effected = Effected.AS_NONE
                elif special_character == '\"' and self.effected is Effected.AS_NONE:
                    self.effected = Effected.AS_DOUBLE_QUOTE_STRING
                elif special_character == '\"' and self.effected is Effected.AS_DOUBLE_QUOTE_STRING:
                    self.effected = Effected.AS_NONE
                elif special_character == '#' and self.effected is Effected.AS_NONE:
                    self.effected = Effected.AS_COMMENT
                elif special_character == '\n' and self.effected is Effected.AS_COMMENT:
                    self.effected = Effected.AS_NONE
                elif special_character == '\n' and self.effected is Effected.AS_DOUBLE_QUOTE_STRING:
                    self.report_error(TokenError.INCOMPLETE_STRING)
                elif special_character == '\n' and self.effected is Effected.AS_SINGLE_QUOTE_STRING:
                    self.report_error(TokenError.INCOMPLETE_STRING)

                if self.buffer != '':
                    self.token = self.buffer
                    # Process for predefined keywords
                    token_processed = self.change_keyword()  # Processed for keyword
                    if not token_processed and self.las_symbol != '.':
                        # If not processed as keywords and the last symbol is not . i.e not a property
                        if self.current_symbol != '' and self.current_symbol == '(':
                            # If current symbol is start bracket then it is function
                            token_processed = self.change_function()
                        else:
                            self.get_next_symbol()
                            if self.next_symbol != '' and self.next_symbol == '(':
                                token_processed = self.change_function()
                    py_line += self.token
                    self.buffer = ''
                    self.token = ''
                py_line += special_character
                self.current_symbol = ''
                if character != " " and character != "\n":
                    self.las_symbol = special_character
            else:
                if self.effected is Effected.AS_NONE:
                    self.buffer += character
                else:
                    py_line += character

        return py_line

    def process(self):

        py_content = ""  # store all the processed line or py_line
        # line_no = 0  # store the characters position
        for line in self.file:
            self.line_no += 1

            # Replace unicode and other javanese character to english.
            line = line.replace("\u202b", "")
            line = line.replace("\u202c", "")
            line = line.replace("꧈", ",")
            line = line.replace("꧉", ".")
            line = line.replace("꧇", "")
            line = line.replace("٪", "%")

            self.line = line

            # Finally appending line to the content of the file
            py_content += self.process_line()

        return py_content



#!/usr/bin/python3
from setuptools import setup
import io
import os

# Package meta-data
NAME = 'sawa'
DESCRIPTION = 'an open source programming language, an interpreter to be precise, where you can write Python code using Javanese character.'
URL = 'https://github.com/lantip/sawa'
EMAIL = 'lantip@underpaidworkers.org'
AUTHOR = 'Lantip'
REQUIRES_PYTHON = '>=3.6.0'
VERSION = None

here = os.path.abspath(os.path.dirname(__file__))

with io.open(os.path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = '\n' + f.read()

# Load the package's __version__.py
about = {}
if not VERSION:
    with open(os.path.join(here, NAME, '__version__.py')) as f:
        exec(f.read(), about)
else:
    about['__version__'] = VERSION

setup(
    name=NAME,
    version=about['__version__'],
    description=DESCRIPTION,
    long_description=long_description,
    long_description_content_type="text/markdown",
    author=AUTHOR,
    author_email=EMAIL,
    python_requires=REQUIRES_PYTHON,
    url=URL,
    packages=['sawa', 'sawa.enum'],
    entry_points={
        'console_scripts': [
            'ꦱꦮ = sawa.cli:run_as_command',
        ],
    },
    license='MIT',
    classifiers=[
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: Implementation :: CPython',
    ],
)